#include <nds.h>
#include "arm7.h"
#include "fw.h"
#include "ipc.h"
#include "../../arm9/include/wifi.h"

int Dump_Card();

//---------------------------------------------------------------------------------
void startSound(int sampleRate, const void* data, u32 bytes, u8 channel, u8 vol,  u8 pan, u8 format) {
//---------------------------------------------------------------------------------
	SCHANNEL_TIMER(channel)  = SOUND_FREQ(sampleRate);
	SCHANNEL_SOURCE(channel) = (u32)data;
	SCHANNEL_LENGTH(channel) = bytes >> 2 ;
	SCHANNEL_CR(channel)     = SCHANNEL_ENABLE | SOUND_ONE_SHOT | SOUND_VOL(vol) | SOUND_PAN(pan) | (format==1?SOUND_8BIT:SOUND_16BIT);
}


//---------------------------------------------------------------------------------
s32 getFreeSoundChannel() {
//---------------------------------------------------------------------------------
	int i;
	for (i=0; i<16; i++) {
		if ( (SCHANNEL_CR(i) & SCHANNEL_ENABLE) == 0 ) return i;
	}
	return -1;
}

void interrupthandler() {
	//u32 flags=REG_IF&REG_IE;
	
	static int heartbeat = 0;

	uint16 but=0, x=0, y=0, xpx=0, ypx=0, z1=0, z2=0, batt=0, aux=0;
	int t1=0, t2=0;
	uint32 temp=0;
	uint8 ct[sizeof(IPC->curtime)];
	u32 i;

	// Update the heartbeat
	heartbeat++;

	// Read the touch screen

	but = REG_KEYXY;

	if (!(but & (1<<6))) {

		touchPosition tempPos = touchReadXY();

		x = tempPos.x;
		y = tempPos.y;
		xpx = tempPos.px;
		ypx = tempPos.py;
	}

	z1 = touchRead(TSC_MEASURE_Z1);
	z2 = touchRead(TSC_MEASURE_Z2);

	
	batt = touchRead(TSC_MEASURE_BATTERY);
	aux  = touchRead(TSC_MEASURE_AUX);

	// Read the time
	rtcGetTime((uint8 *)ct);
	BCDToInteger((uint8 *)&(ct[1]), 7);

	// Read the temperature
	temp = touchReadTemperature(&t1, &t2);

	// Update the IPC struct
	IPC->heartbeat	= heartbeat;
	IPC->buttons		= but;
	IPC->touchX			= x;
	IPC->touchY			= y;
	IPC->touchXpx		= xpx;
	IPC->touchYpx		= ypx;
	IPC->touchZ1		= z1;
	IPC->touchZ2		= z2;
	IPC->battery		= batt;
	IPC->aux			= aux;

	for(i=0; i<sizeof(ct); i++) {
		IPC->curtime[i] = ct[i];
	}
	IPC->temperature = temp;
	IPC->tdiode1 = t1;
	IPC->tdiode2 = t2;
	
	//sound code  :)
	TransferSound *snd = IPC->soundData;
	IPC->soundData = 0;

	if (0 != snd) {

		for (i=0; i<snd->count; i++) {
			s32 chan = getFreeSoundChannel();

			if (chan >= 0) {
				startSound(snd->data[i].rate, snd->data[i].data, snd->data[i].len, chan, snd->data[i].vol, snd->data[i].pan, snd->data[i].format);
			}
		}
	}
	
	
	//VBLANK_INTR_WAIT_FLAGS|=flags;
	//REG_IF=flags;
}

//change power management register
void writePM(int flags) {
	call3 spi_write=(call3)0x2369;
	spi_write(SPI_DEVICE_POWER,0,1);
	spi_write(SPI_DEVICE_POWER,flags,0);
}

//shut down, wait for lid to open
void arm7sleep() {
	u32 ie_save;
	writePM(PM_TOPLIGHT | PM_BOTTOMLIGHT | PM_SOUND | PM_LED_SLOW);
	swiChangeSoundBias(0,0x400);
	ie_save=REG_IE;
	REG_IE=IRQ_LID;
	swiSleep();
	REG_IF=~0;
	REG_IE=ie_save;
	writePM(PM_TOPLIGHT | PM_BOTTOMLIGHT | PM_SOUND);
	swiChangeSoundBias(1,0x400);
}

static void clearmem(u32 dest, u32 size) {
	int i=0;
	swiFastCopy(&i, (u32*)dest, 0x01000000 | (size>>2));
}

void hardware_init() {
	REG_IME=REG_IE=0;
	
	writePM(PM_TOPLIGHT | PM_BOTTOMLIGHT | PM_SOUND);
	POWER_CR=POWER_SOUND;
	swiChangeSoundBias(1,0x400);

	//reset DMA
	clearmem(0x40000B0, 0x30);

	//((call0)0x119d)();
}

void printtext(int line, char *str)
{
	char *s=str;
	u16 *p=(u16*)(0x6200000+line*64);
	while (*s) *p++=*s++;
} 

int main() {
	u32 i;

	IPC_ARM9=-1;
	IPC_XKEYS=0;
	hardware_init();

	DISP_SR=DISP_VBLANK_IRQ;
	/*IRQ_HANDLER=interrupthandler;
	REG_IE=IRQ_VBLANK;
	REG_IF=~0;
	REG_IME=IME_ENABLE;*/
	
	irqInit();
	irqSet(IRQ_VBLANK, interrupthandler);
	irqEnable(IRQ_VBLANK);
	

	load_PersonalData();
	load_WifiData(0,(u32*)WIFI_SETTING);
	
	
	while(1) {
		swiWaitForVBlank();

		IPC_XKEYS=REG_KEYXY;
		
		i=IPC_ARM9;
		if(i!=-1) {
			swiWaitForVBlank();
			if(i!=IPC_ARM9)
			{
				IPC_ARM7 = -2;
				IPC_ARM9 = -1;
				continue;
			}


			IPC_ARM9=-1;
			switch(i) {
			
				case IPC_NDS:
					if(load_card())
					{
						hardware_init();
						IPC_ARM7=IPC_NDS_GO;
						(*(call0*)0x27ffe34)();
					}
					else
					{
						IPC_ARM7=IPC_NDS_STOP;
						break;
					}
					
				case IPC_PASSME:
					hardware_init();
					((call0)0x80000c0)();
					
				case IPC_GBA:
					if(PersonalData->gbaScreen)
						i=PM_BOTTOMLIGHT | PM_SOUND;
					else
						i=PM_TOPLIGHT | PM_SOUND;
					writePM(i);
					swiChangeSoundBias(0,0x400);
					((call0)0x11a4)();
					
				case IPC_SLEEP:
					arm7sleep();
					break;
					
				case IPC_WRITE_PSETTINGS:
					save_PersonalData();
					break;
					
				case IPC_POWEROFF:
					writePM(PM_OFF);
					break;
					
				/*case IPC_READ_WIFI:
					load_WifiData(IPC_ARM9_P1,(u32*)WIFI_SETTING);
					IPC_ARM7=IPC_READ_WIFI;
					IPC_ARM9=IPC_ARM9_P1-1;
					break;*/
					
				case IPC_WRITE_WIFI_0:
					save_WifiData(0,(u32*)WIFI_SETTING);
					break;
				case IPC_WRITE_WIFI_1:
					save_WifiData(1,(u32*)(WIFI_SETTING+0x100));
					break;
				case IPC_WRITE_WIFI_2:
					save_WifiData(2,(u32*)(WIFI_SETTING+0x200));
					break;
					
				#ifdef DS_CARD_DUMP
				case IPC_DUMPCARD:
					Dump_Card();
					IPC_ARM7=IPC_DUMPCARD;
					break;
				#endif
			}
		}
	}
	return 0;
}

